-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD: Digital Circuits and Systems - DIGSYS         
--------------------------------------------------------------------------------
-- P6. Example tutorial. laboratory prototype: Flashing torche project 
-- Target board: Xilinx CoolRunner-II 
-- Target PLD: XC2C256-TQ144-7

-- Plan: An schematic based on a FSM and some additional components. 
-- It includes the CLK geenrator to obtain both the 1 kHz system CLK and the 
-- signal for flashing the lamp from the board 8 MHz oscillator   

-- Ref: http://digsys.upc.es/csd/P06/P6_T/Lamp/Lamp.html
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
			
ENTITY Flashing_Lamp IS
	Port (
		OSC_CLK_in	: IN 	std_logic;
		CD_L			: IN	STD_LOGIC; -- Let's connect the CD to a push-button
		ON_B_L	: IN	STD_LOGIC;      
		Z_L		: OUT	STD_LOGIC;

-- The internal state of the FSM is wired to port pins for debugging purposes.     
		Current_state_L	: OUT	STD_LOGIC_VECTOR (2 DOWNTO 0);
      -- Note that if coded in one-hot it has to be (5 downto 0)    

		-- for monitoring the CLK signals in the oscilloscope      
		CLK_5Hz_SQ_out	: OUT	STD_LOGIC;      
		CLK_1kHz_SQ_out	: OUT	STD_LOGIC;      
		OSC_CLK_out		: OUT	STD_LOGIC     
   );

END Flashing_Lamp;

	-- The desciption of the proposed schematic
	-- 
ARCHITECTURE schematic OF Flashing_Lamp IS

	COMPONENT Lamp_Control IS -- This is the FSM
	      Port (	CLK 	: IN	STD_LOGIC;
					ON_B		: IN	STD_LOGIC;
					CD   		: IN 	STD_LOGIC;
					States	: OUT	STD_LOGIC_VECTOR(2 DOWNTO 0);
					S 			: OUT	STD_LOGIC_VECTOR(1 DOWNTO 0)
		);
	END COMPONENT;

	COMPONENT MUX4 IS  	-- To select different kind of outputs
		port(	E_L 	:	in std_logic;
				Ch0		:	in std_logic;
				Ch1		:	in std_logic;
				Ch2		:	in std_logic;
				Ch3		:	in std_logic;
				Sel		:	in std_logic_vector(1 downto 0);
				y		:	out std_logic
			);
	END COMPONENT;

	COMPONENT CLK_Generator IS  
	      PORT(	CD,CE			: 	IN  std_logic;
				OSC_CLK_in		: 	IN  std_logic;
				OSC_CLK_out		:	OUT std_logic;
				CLK_1kHz_SQ		:	OUT std_logic;
				CLK_5Hz_SQ		:	OUT std_logic
	          );
	end COMPONENT;       
	
-- Signals for connecting components together (the wires)
	SIGNAL CD				: std_logic;
	SIGNAL ON_B				: std_logic;
	SIGNAL Z				: std_logic;
	SIGNAL CLK_1kHz_SQ		: std_logic;
	SIGNAL CLK_5Hz_SQ		: std_logic;
	SIGNAL Sel				: std_logic_vector (1 downto 0);
	SIGNAL Current_state	: std_logic_vector (2 downto 0);
	

	BEGIN  

-- Instantiation of components
	Chip1 : Lamp_Control        
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK		=> CLK_1kHz_SQ,
			CD		=> CD,
			ON_B	=> ON_B,
			States	=> Current_state,
			S		=> Sel	
		);

	Chip2 : MUX4        
-- 	from component name => to signal or port name	
		PORT MAP (
			E_L		=> '0',
			Ch0		=> '0',
			Ch1		=> '1',
			Ch2	  	=> CLK_5Hz_SQ,
			Ch3		=> '0', -- not used
			Sel	  	=> Sel,
 			Y		=> Z
		);          

	Chip3 : CLK_Generator        
-- 	from component name => to signal or port name	
		PORT MAP (
			CD			=> CD,
			CE			=> '1',
			OSC_CLK_in	=> OSC_CLK_in,
			OSC_CLK_out	=> OSC_CLK_out,
			CLK_1kHz_SQ	=> CLK_1kHz_SQ, 
			CLK_5Hz_SQ	=> CLK_5Hz_SQ
		);          
  
-- extra buufers to adapt the board hardware
   
CD <= not (CD_L); -- The board has active-low push-buttons
ON_B <= not (ON_B_L); 
   
Z_L <= not (Z); -- the board has active-low LED's
Current_state_L <= not (Current_state);

CLK_1kHz_SQ_out <= CLK_1kHz_SQ;
CLK_5Hz_SQ_out 	<= CLK_5Hz_SQ;

END schematic;






